==> smile-core-2.6.0-sources.jar: smile/validation/metric/MAD.java <==
/*******************************************************************************
 * Copyright (c) 2010-2020 Haifeng Li. All rights reserved.
 *
 * Smile is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * Smile is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Smile.  If not, see <https://www.gnu.org/licenses/>.
 ******************************************************************************/

package smile.validation.metric;

/**
 * Mean absolute deviation error.
 * 
 * @author Haifeng Li
 */
public class MAD implements RegressionMetric {
    private static final long serialVersionUID = 2L;
    /** Default instance. */
    public final static MAD instance = new MAD();

    @Override
    public double score(double[] truth, double[] prediction) {
        return of(truth, prediction);
    }

    /** Calculates the mean absolute deviation error. */
    public static double of(double[] truth, double[] prediction) {
        if (truth.length != prediction.length) {
            throw new IllegalArgumentException(String.format("The vector sizes don't match: %d != %d.", truth.length, prediction.length));
        }

        int n = truth.length;
        double error = 0.0;
        for (int i = 0; i < n; i++) {
            error += Math.abs(truth[i] - prediction[i]);
        }

        return error/n;
    }

    @Override
    public String toString() {
        return "MAD";
    }
}

==> smile-core-3.1.0-sources.jar: smile/validation/metric/MAD.java <==
/*
 * Copyright (c) 2010-2021 Haifeng Li. All rights reserved.
 *
 * Smile is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Smile is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Smile.  If not, see <https://www.gnu.org/licenses/>.
 */

package smile.validation.metric;

/**
 * Mean absolute deviation error.
 * 
 * @author Haifeng Li
 */
public class MAD implements RegressionMetric {
    private static final long serialVersionUID = 2L;
    /** Default instance. */
    public final static MAD instance = new MAD();

    @Override
    public double score(double[] truth, double[] prediction) {
        return of(truth, prediction);
    }

    /**
     * Calculates the mean absolute deviation error.
     * @param truth the ground truth.
     * @param prediction the prediction.
     * @return the metric.
     */
    public static double of(double[] truth, double[] prediction) {
        if (truth.length != prediction.length) {
            throw new IllegalArgumentException(String.format("The vector sizes don't match: %d != %d.", truth.length, prediction.length));
        }

        int n = truth.length;
        double error = 0.0;
        for (int i = 0; i < n; i++) {
            error += Math.abs(truth[i] - prediction[i]);
        }

        return error/n;
    }

    @Override
    public String toString() {
        return "MAD";
    }
}
